/********************************************************************************
INT_Ch.c:   Multi-channels as rising or falling edge interrupt in single 
            slot. 
Compiler: BC++ 3.1, 
          Turbo C ++ 1.01(3.01) (free from http://community.borland.com/museum)
          MSC 6.0,
          MSVC 1.52.
          
Compile mode: large

Project: INT_Ch.c
         ..\..\..\..\Lib\8000A.Lib
  
To use 8048W on iPAC-8000 system
	step1: call i8048W_Init 
	step2: call InstallSlotLevelIsr(slot,ISR_Function)to install your own ISR function
	step3: to enable rising/falling edge trigger for each channel
	step4: define interrupt service for each events.
	step5: call i8048W_UnFreezeCPU disable backplane interrupt register.
	step6: call UnInstallSlotLevelIsr to uninstall the ISR function.
	

Note: To test 8048W interrupt ISR function on iPAC-8000 system, 
1. You must avoid to use Print, GetCh and Kbhit , because these function themselves will produce interrupt also.   
2. If use UDP version os image, you must use closeudp command to close udp under prompt C837_V2_UDP>closeudp

[4 Sep,2005] by Martin                                                              
*********************************************************************************/
#include <stdlib.h>
#include <conio.h>
#include "..\..\..\Lib\8000A.h"
#include "..\..\..\lib\8048W.h"

  

void ISR_Function(int slot);
void showBasicInfo(int slot);
void main(void)
{   

    int iSlot;  
    int slot,channel;
    int data;
    long findIO=0;
    int quit=0;

    InitLib();
    
    
    InstallCom_1(115200L,8,0,1);
	for(slot=0;slot<NumberOfSlot;slot++)
	{
		if(NameOfModule[slot]==48 && (ModuleType[slot] & _PARALLEL))
		{
    		i8048W_Init(slot);
    		printCom_1("find %s in slot %d\r\n",ModuleFullName[slot],slot);
    		
    		showBasicInfo(slot);
    		
    		printCom_1("Press 'q' to exit program\r\n");
    		printCom_1("Press any key to read Event Count\r\n");
    		
    		
		    //InstallSlotLevelIsr(slot,ISR_Function);
		    InstallSlotRisingIsr(slot,ISR_Function);
			
		    i8048W_Set_RisingReg(slot,0,1);     //0: Channel 0     1:Enable     
		    
		    ////Enable channel 0,1,2 as Falling edge interrupt. 
		    i8048W_Set_FallingReg(slot,0,1);    //0: Channel 0 , 1:Enable      
		    i8048W_Set_FallingReg(slot,1,1);    //0: Channel 1 , 1:Enable     
		    i8048W_Set_FallingReg(slot,2,1);    //0: Channel 2 , 1:Enable  
		    findIO=1;
		    iSlot = slot;
		    break;
		}
	}
	if(!findIO)
	{
		printCom_1("Can not find 8048W\r\n");
		Delay(10);
		RestoreCom_1(); 		
		exit(0);
	}
	while(!quit)
	{
		
		if (IsCom_1())
		{
			data=ReadCom_1();
			switch(data)
			{
				case 'q':				
					quit=1;
					break;				
				default:	
			        printCom_1("Rising S%dC0==%lu    Falling S%dC0==%lu S%dC1==%lu S%dC2==%lu\n\r"      
                        ,iSlot,i8048W_RisingEventCount[iSlot][0]
                        ,iSlot,i8048W_FallingEventCount[iSlot][0]
                        ,iSlot,i8048W_FallingEventCount[iSlot][1]                     
                        ,iSlot,i8048W_FallingEventCount[iSlot][2]);
					break;
			}
		}
		
	} 	
	i8048W_UnFreezeCPU(iSlot);
	
	UnInstallSlotLevelIsr(iSlot);
	RestoreCom_1();                         
    
                                    
}

/********************************************************
Please don's use the Print or printCom1 function in ISR,
these function will cause another interrupt problem. 
*********************************************************/
void ISR_Function(int slot)
{
	
    if(i8048W_Read_RisingEvent(slot,0))
    {
        //Add user's ISR code for channel0.
    }
    
    if(i8048W_Read_FallingEvent(slot,0))                         
    {                                       
        //Add user's ISR code for channel0. 
    }                                       
    
    if(i8048W_Read_FallingEvent(slot,1))
    {
        //Add user's ISR code for channel1. 
    }
    
    if(i8048W_Read_FallingEvent(slot,2))
    {
        //Add user's ISR code for channel2.         
    }

 
}   
    
    
void showBasicInfo(int slot)
{
	char dateStr[32];
	i8048W_GetLibDate(dateStr);
	printCom_1("*******************************************************\n");
	printCom_1("\tLibrary build date === %s\n",dateStr);
	printCom_1("\tLibrary version === %04X\n",i8048W_GetLibVersion());
	printCom_1("\tVersion of Lattice Firmware at Slot %d === %02X \n",slot,i8048W_GetFirmwareVersion(slot));
	printCom_1("\n*******************************************************\n");
}